"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decorateDiagnostics = void 0;
const utils_1 = require("../utils");
const sveltekit_1 = require("./sveltekit");
function decorateDiagnostics(ls, info, typescript, logger) {
    decorateSyntacticDiagnostics(ls, info, typescript, logger);
    decorateSemanticDiagnostics(ls, info, typescript, logger);
    decorateSuggestionDiagnostics(ls, info, typescript, logger);
}
exports.decorateDiagnostics = decorateDiagnostics;
function decorateSyntacticDiagnostics(ls, info, typescript, logger) {
    const getSyntacticDiagnostics = ls.getSyntacticDiagnostics;
    ls.getSyntacticDiagnostics = (fileName) => {
        // Diagnostics inside Svelte files are done
        // by the svelte-language-server / Svelte for VS Code extension
        if ((0, utils_1.isSvelteFilePath)(fileName)) {
            return [];
        }
        const kitDiagnostics = getKitDiagnostics('getSyntacticDiagnostics', fileName, info, typescript, logger);
        return kitDiagnostics !== null && kitDiagnostics !== void 0 ? kitDiagnostics : getSyntacticDiagnostics(fileName);
    };
}
function decorateSemanticDiagnostics(ls, info, typescript, logger) {
    const getSemanticDiagnostics = ls.getSemanticDiagnostics;
    ls.getSemanticDiagnostics = (fileName) => {
        // Diagnostics inside Svelte files are done
        // by the svelte-language-server / Svelte for VS Code extension
        if ((0, utils_1.isSvelteFilePath)(fileName)) {
            return [];
        }
        const kitDiagnostics = getKitDiagnostics('getSemanticDiagnostics', fileName, info, typescript, logger);
        return kitDiagnostics !== null && kitDiagnostics !== void 0 ? kitDiagnostics : getSemanticDiagnostics(fileName);
    };
}
function decorateSuggestionDiagnostics(ls, info, typescript, logger) {
    const getSuggestionDiagnostics = ls.getSuggestionDiagnostics;
    ls.getSuggestionDiagnostics = (fileName) => {
        // Diagnostics inside Svelte files are done
        // by the svelte-language-server / Svelte for VS Code extension
        if ((0, utils_1.isSvelteFilePath)(fileName)) {
            return [];
        }
        const kitDiagnostics = getKitDiagnostics('getSuggestionDiagnostics', fileName, info, typescript, logger);
        return kitDiagnostics !== null && kitDiagnostics !== void 0 ? kitDiagnostics : getSuggestionDiagnostics(fileName);
    };
}
function getKitDiagnostics(methodName, fileName, info, ts, logger) {
    const result = (0, sveltekit_1.getVirtualLS)(fileName, info, ts, logger);
    if (!result)
        return;
    const { languageService, toOriginalPos } = result;
    const diagnostics = [];
    for (let diagnostic of languageService[methodName](fileName)) {
        if (!diagnostic.start || !diagnostic.length) {
            diagnostics.push(diagnostic);
            continue;
        }
        const mapped = toOriginalPos(diagnostic.start);
        if (mapped.inGenerated) {
            // If not "Cannot find module './$types' .." then filter out
            if (diagnostic.code !== 2307) {
                continue;
            }
            else {
                diagnostic = {
                    ...diagnostic,
                    // adjust length so it doesn't spill over to the next line
                    length: 1,
                    messageText: typeof diagnostic.messageText === 'string' &&
                        diagnostic.messageText.includes('./$types')
                        ? diagnostic.messageText +
                            ` (this likely means that SvelteKit's type generation didn't run yet)`
                        : diagnostic.messageText
                };
            }
        }
        diagnostic = {
            ...diagnostic,
            start: mapped.pos
        };
        diagnostics.push(diagnostic);
    }
    // @ts-ignore TS doesn't get the return type right
    return diagnostics;
}
//# sourceMappingURL=diagnostics.js.map