import type ts from 'typescript/lib/tsserverlibrary';
import { Logger } from '../logger';
type _ts = typeof ts;
interface KitSnapshot {
    file: ts.IScriptSnapshot;
    version: string;
    addedCode: Array<{
        generatedPos: number;
        originalPos: number;
        length: number;
        total: number;
        inserted: string;
    }>;
}
export declare const kitExports: {
    prerender: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    ssr: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    csr: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    trailingSlash: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    config: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    actions: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
    load: {
        displayParts: {
            text: string;
            kind: string;
        }[];
        documentation: {
            text: string;
            kind: string;
        }[];
    };
};
export declare function getProxiedLanguageService(info: ts.server.PluginCreateInfo, ts: _ts, logger?: Logger): {
    languageService: ts.LanguageService;
    languageServiceHost: ts.LanguageServiceHost & {
        getKitScriptSnapshotIfUpToDate: (fileName: string) => KitSnapshot | undefined;
        upsertKitFile: (fileName: string) => void;
    };
} | {
    languageService: ts.LanguageService;
    languageServiceHost: {
        files: Record<string, KitSnapshot>;
        log(): void;
        trace(): void;
        error(): void;
        getCompilationSettings(): ts.CompilerOptions;
        getScriptIsOpen(): boolean;
        getCurrentDirectory(): string;
        getDefaultLibFileName(o: any): string;
        resolveModuleNames: ((...args: any[]) => (ts.ResolvedModule | undefined)[]) | undefined;
        resolveModuleNameLiterals: ((...args: any[]) => readonly ts.ResolvedModuleWithFailedLookupLocations[]) | undefined;
        getScriptVersion(fileName: string): string;
        getScriptSnapshot(fileName: string): ts.IScriptSnapshot | undefined;
        getScriptFileNames(): string[];
        getKitScriptSnapshotIfUpToDate(fileName: string): KitSnapshot | undefined;
        upsertKitFile(fileName: string): KitSnapshot | undefined;
        readFile(fileName: string): string | undefined;
        fileExists(fileName: string): boolean;
    };
};
export declare function getVirtualLS(fileName: string, info: ts.server.PluginCreateInfo, ts: _ts, logger?: Logger): {
    languageService: ts.LanguageService;
    addedCode: {
        generatedPos: number;
        originalPos: number;
        length: number;
        total: number;
        inserted: string;
    }[];
    toVirtualPos: (pos: number) => number;
    toOriginalPos: (pos: number) => {
        pos: number;
        inGenerated: boolean;
    };
} | undefined;
export declare function toVirtualPos(pos: number, addedCode: KitSnapshot['addedCode']): number;
export declare function toOriginalPos(pos: number, addedCode: KitSnapshot['addedCode']): {
    pos: number;
    inGenerated: boolean;
};
export {};
