"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toOriginalPos = exports.toVirtualPos = exports.getVirtualLS = exports.getProxiedLanguageService = exports.kitExports = void 0;
const path_1 = __importDefault(require("path"));
const cache = new WeakMap();
exports.kitExports = {
    prerender: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'prerender',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'boolean',
                kind: 'keyword'
            },
            {
                text: ' | ',
                kind: 'punctuation'
            },
            {
                text: "'auto'",
                kind: 'stringLiteral'
            }
        ],
        documentation: [
            {
                text: 'Control whether or not this page is prerendered. More info: https://kit.svelte.dev/docs/page-options#prerender',
                kind: 'text'
            }
        ]
    },
    ssr: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'ssr',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'boolean',
                kind: 'keyword'
            }
        ],
        documentation: [
            {
                text: 'Control whether or not this page is server-side rendered. More info: https://kit.svelte.dev/docs/page-options#ssr',
                kind: 'text'
            }
        ]
    },
    csr: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'csr',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'boolean',
                kind: 'keyword'
            }
        ],
        documentation: [
            {
                text: 'Control whether or not this page is hydrated (i.e. if JS is delivered to the client). More info: https://kit.svelte.dev/docs/page-options#csr',
                kind: 'text'
            }
        ]
    },
    trailingSlash: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'trailingSlash',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: "'auto' | 'always' | 'never'",
                kind: 'stringLiteral'
            }
        ],
        documentation: [
            {
                text: 'Control how SvelteKit should handle (missing) trailing slashes in the URL. More info: https://kit.svelte.dev/docs/page-options#trailingslash',
                kind: 'text'
            }
        ]
    },
    config: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'config',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'Config',
                kind: 'interfaceName'
            }
        ],
        documentation: [
            {
                text: `With the concept of adapters, SvelteKit is able to run on a variety of platforms. ` +
                    `Each of these might have specific configuration to further tweak the deployment, which you can configure here. ` +
                    `More info: https://kit.svelte.dev/docs/page-options#config`,
                kind: 'text'
            }
        ]
    },
    actions: {
        displayParts: [
            {
                text: 'const',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'actions',
                kind: 'localName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'Actions',
                kind: 'interfaceName'
            }
        ],
        documentation: [
            {
                text: `An object of methods which handle form POST requests. ` +
                    `More info: https://kit.svelte.dev/docs/form-actions`,
                kind: 'text'
            }
        ]
    },
    load: {
        displayParts: [
            {
                text: 'export',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'function',
                kind: 'keyword'
            },
            {
                text: ' ',
                kind: 'space'
            },
            {
                text: 'load',
                kind: 'localName'
            },
            {
                text: '(',
                kind: 'punctuation'
            },
            {
                text: 'event',
                kind: 'parameterName'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'LoadEvent',
                kind: 'interfaceName'
            },
            {
                text: ')',
                kind: 'punctuation'
            },
            {
                text: ': ',
                kind: 'punctuation'
            },
            {
                text: 'Promise',
                kind: 'keyword'
            },
            {
                text: '<',
                kind: 'punctuation'
            },
            {
                text: 'LoadOutput',
                kind: 'interfaceName'
            },
            {
                text: '>',
                kind: 'punctuation'
            }
        ],
        documentation: [
            {
                text: 'Loads data for the given page or layout. More info: https://kit.svelte.dev/docs/load',
                kind: 'text'
            }
        ]
    }
};
function getProxiedLanguageService(info, ts, logger) {
    const cachedProxiedLanguageService = cache.get(info);
    if (cachedProxiedLanguageService) {
        return cachedProxiedLanguageService;
    }
    const originalLanguageServiceHost = info.languageServiceHost;
    class ProxiedLanguageServiceHost {
        constructor() {
            this.files = {};
            this.resolveModuleNames = originalLanguageServiceHost.resolveModuleNames
                ? (...args) => {
                    return originalLanguageServiceHost.resolveModuleNames(
                    // @ts-ignore
                    ...args);
                }
                : undefined;
            this.resolveModuleNameLiterals = originalLanguageServiceHost.resolveModuleNameLiterals
                ? (...args) => {
                    return originalLanguageServiceHost.resolveModuleNameLiterals(
                    // @ts-ignore
                    ...args);
                }
                : undefined;
        }
        log() { }
        trace() { }
        error() { }
        getCompilationSettings() {
            return originalLanguageServiceHost.getCompilationSettings();
        }
        getScriptIsOpen() {
            return true;
        }
        getCurrentDirectory() {
            return originalLanguageServiceHost.getCurrentDirectory();
        }
        getDefaultLibFileName(o) {
            return originalLanguageServiceHost.getDefaultLibFileName(o);
        }
        getScriptVersion(fileName) {
            const file = this.files[fileName];
            if (!file)
                return originalLanguageServiceHost.getScriptVersion(fileName);
            return file.version.toString();
        }
        getScriptSnapshot(fileName) {
            const file = this.files[fileName];
            if (!file)
                return originalLanguageServiceHost.getScriptSnapshot(fileName);
            return file.file;
        }
        getScriptFileNames() {
            const names = new Set(Object.keys(this.files));
            const files = originalLanguageServiceHost.getScriptFileNames();
            for (const file of files) {
                names.add(file);
            }
            return [...names];
        }
        getKitScriptSnapshotIfUpToDate(fileName) {
            if (!this.files[fileName] ||
                this.getScriptVersion(fileName) !==
                    originalLanguageServiceHost.getScriptVersion(fileName)) {
                return undefined;
            }
            return this.files[fileName];
        }
        upsertKitFile(fileName) {
            var _a;
            const basename = path_1.default.basename(fileName);
            if (!kitPageFiles.has(basename))
                return;
            const source = (_a = info.languageService.getProgram()) === null || _a === void 0 ? void 0 : _a.getSourceFile(fileName);
            if (!source)
                return;
            const addedCode = [];
            const insert = (pos, inserted) => {
                var _a, _b, _c, _d;
                const insertionIdx = addedCode.findIndex((c) => c.generatedPos > pos);
                if (insertionIdx >= 0) {
                    for (let i = insertionIdx; i < addedCode.length; i++) {
                        addedCode[i].generatedPos += inserted.length;
                        addedCode[i].total += inserted.length;
                    }
                    const prevTotal = (_b = (_a = addedCode[insertionIdx - 1]) === null || _a === void 0 ? void 0 : _a.total) !== null && _b !== void 0 ? _b : 0;
                    addedCode.splice(insertionIdx, 0, {
                        generatedPos: pos + prevTotal,
                        originalPos: pos,
                        length: inserted.length,
                        inserted,
                        total: prevTotal + inserted.length
                    });
                }
                else {
                    const prevTotal = (_d = (_c = addedCode[addedCode.length - 1]) === null || _c === void 0 ? void 0 : _c.total) !== null && _d !== void 0 ? _d : 0;
                    addedCode.push({
                        generatedPos: pos + prevTotal,
                        originalPos: pos,
                        length: inserted.length,
                        inserted,
                        total: prevTotal + inserted.length
                    });
                }
            };
            const isTsFile = basename.endsWith('.ts');
            const exports = findExports(ts, source, isTsFile);
            // add type to load function if not explicitly typed
            const load = exports.get('load');
            if ((load === null || load === void 0 ? void 0 : load.type) === 'function' &&
                load.node.parameters.length === 1 &&
                !load.hasTypeDefinition) {
                const pos = load.node.parameters[0].getEnd();
                const inserted = `: import('./$types').${basename.includes('layout') ? 'Layout' : 'Page'}${basename.includes('server') ? 'Server' : ''}LoadEvent`;
                insert(pos, inserted);
            }
            // add type to actions variable if not explicitly typed
            const actions = exports.get('actions');
            if ((actions === null || actions === void 0 ? void 0 : actions.type) === 'var' && !actions.hasTypeDefinition && actions.node.initializer) {
                const pos = actions.node.initializer.getEnd();
                const inserted = ` satisfies import('./$types').Actions`;
                insert(pos, inserted);
            }
            // add type to prerender variable if not explicitly typed
            const prerender = exports.get('prerender');
            if ((prerender === null || prerender === void 0 ? void 0 : prerender.type) === 'var' &&
                !prerender.hasTypeDefinition &&
                prerender.node.initializer) {
                const pos = prerender.node.name.getEnd();
                const inserted = ` : import('@sveltejs/kit').PrerenderOption`;
                insert(pos, inserted);
            }
            // add type to trailingSlash variable if not explicitly typed
            const trailingSlash = exports.get('trailingSlash');
            if ((trailingSlash === null || trailingSlash === void 0 ? void 0 : trailingSlash.type) === 'var' &&
                !trailingSlash.hasTypeDefinition &&
                trailingSlash.node.initializer) {
                const pos = trailingSlash.node.name.getEnd();
                const inserted = ` : 'never' | 'always' | 'ignore'`; // TODO this should be exported from kit
                insert(pos, inserted);
            }
            // add type to ssr variable if not explicitly typed
            const ssr = exports.get('ssr');
            if ((ssr === null || ssr === void 0 ? void 0 : ssr.type) === 'var' && !ssr.hasTypeDefinition && ssr.node.initializer) {
                const pos = ssr.node.name.getEnd();
                const inserted = ` : boolean`;
                insert(pos, inserted);
            }
            // add type to csr variable if not explicitly typed
            const csr = exports.get('csr');
            if ((csr === null || csr === void 0 ? void 0 : csr.type) === 'var' && !csr.hasTypeDefinition && csr.node.initializer) {
                const pos = csr.node.name.getEnd();
                const inserted = ` : boolean`;
                insert(pos, inserted);
            }
            // construct generated text from internal text and addedCode array
            const originalText = source.getFullText();
            let pos = 0;
            let text = '';
            for (const added of addedCode) {
                text += originalText.slice(pos, added.originalPos) + added.inserted;
                pos = added.originalPos;
            }
            text += originalText.slice(pos);
            const snap = ts.ScriptSnapshot.fromString(text);
            snap.getChangeRange = (_) => undefined;
            this.files[fileName] = {
                version: originalLanguageServiceHost.getScriptVersion(fileName),
                file: snap,
                addedCode
            };
            return this.files[fileName];
        }
        readFile(fileName) {
            const file = this.files[fileName];
            return file
                ? file.file.getText(0, file.file.getLength())
                : originalLanguageServiceHost.readFile(fileName);
        }
        fileExists(fileName) {
            return (this.files[fileName] !== undefined ||
                originalLanguageServiceHost.fileExists(fileName));
        }
    }
    const languageServiceHost = new ProxiedLanguageServiceHost();
    const languageService = ts.createLanguageService(languageServiceHost, ts.createDocumentRegistry());
    cache.set(info, { languageService, languageServiceHost });
    return {
        languageService,
        languageServiceHost
    };
}
exports.getProxiedLanguageService = getProxiedLanguageService;
const kitPageFiles = new Set([
    '+page.ts',
    '+page.js',
    '+layout.ts',
    '+layout.js',
    '+page.server.ts',
    '+page.server.js',
    '+layout.server.ts',
    '+layout.server.js'
]);
function getVirtualLS(fileName, info, ts, logger) {
    var _a;
    const proxy = getProxiedLanguageService(info, ts, logger);
    const result = (_a = proxy.languageServiceHost.getKitScriptSnapshotIfUpToDate(fileName)) !== null && _a !== void 0 ? _a : proxy.languageServiceHost.upsertKitFile(fileName);
    if (result) {
        return {
            languageService: proxy.languageService,
            addedCode: result.addedCode,
            toVirtualPos: (pos) => toVirtualPos(pos, result.addedCode),
            toOriginalPos: (pos) => toOriginalPos(pos, result.addedCode)
        };
    }
}
exports.getVirtualLS = getVirtualLS;
function toVirtualPos(pos, addedCode) {
    let total = 0;
    for (const added of addedCode) {
        if (pos < added.originalPos)
            break;
        total += added.length;
    }
    return pos + total;
}
exports.toVirtualPos = toVirtualPos;
function toOriginalPos(pos, addedCode) {
    let total = 0;
    let idx = 0;
    for (; idx < addedCode.length; idx++) {
        const added = addedCode[idx];
        if (pos < added.generatedPos)
            break;
        total += added.length;
    }
    if (idx > 0) {
        const prev = addedCode[idx - 1];
        // If pos is in the middle of an added range, return the start of the addition
        if (pos > prev.generatedPos && pos < prev.generatedPos + prev.length) {
            return { pos: prev.originalPos, inGenerated: true };
        }
    }
    return { pos: pos - total, inGenerated: false };
}
exports.toOriginalPos = toOriginalPos;
/**
 * Finds the top level const/let/function exports of a source file.
 */
function findExports(ts, source, isTsFile) {
    var _a, _b, _c, _d, _e;
    const exports = new Map();
    // TODO handle indirect exports?
    for (const statement of source.statements) {
        if (ts.isFunctionDeclaration(statement) &&
            statement.name &&
            ((_b = (_a = ts.getModifiers(statement)) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.kind) === ts.SyntaxKind.ExportKeyword) {
            // export function x ...
            exports.set(statement.name.text, {
                type: 'function',
                node: statement,
                hasTypeDefinition: !!((_c = statement.parameters[0]) === null || _c === void 0 ? void 0 : _c.type) ||
                    (!isTsFile &&
                        (!!ts.getJSDocType(statement) ||
                            (statement.parameters[0] &&
                                !!ts.getJSDocParameterTags(statement.parameters[0]).length)))
            });
        }
        if (ts.isVariableStatement(statement) &&
            statement.declarationList.declarations.length === 1 &&
            ((_e = (_d = ts.getModifiers(statement)) === null || _d === void 0 ? void 0 : _d[0]) === null || _e === void 0 ? void 0 : _e.kind) === ts.SyntaxKind.ExportKeyword) {
            // export const x = ...
            const declaration = statement.declarationList.declarations[0];
            const hasTypeDefinition = !!declaration.type ||
                (!isTsFile && !!ts.getJSDocType(declaration)) ||
                (!!declaration.initializer && ts.isSatisfiesExpression(declaration.initializer));
            if (declaration.initializer &&
                (ts.isFunctionExpression(declaration.initializer) ||
                    ts.isArrowFunction(declaration.initializer))) {
                exports.set(declaration.name.getText(), {
                    type: 'function',
                    node: declaration.initializer,
                    hasTypeDefinition
                });
            }
            else {
                exports.set(declaration.name.getText(), {
                    type: 'var',
                    node: declaration,
                    hasTypeDefinition
                });
            }
        }
    }
    return exports;
}
//# sourceMappingURL=sveltekit.js.map